
------------------
-- // Ghost \\ --
------------------
local Ghost = {}
Ghost.__index = Ghost


------------------------
-- // Declarations \\ --
------------------------

--Shortcuts
  lkid = love.keyboard.isDown
  gfx = love.graphics
  
--Locals
  local _showGhostHitbox = false

  local colors = {
    {255, 255, 255, 255},
    {30, 190, 30, 255},
    {190, 30, 30, 255},
    }

function Ghost.new(o)
  local self = o or {}
  self.name         = "ghost"
  
  self.sprites = {
   
    idle = gfx.newImage("gfx/ghost_idle.png")
    
  }
  
  self.scale        = self.scale or 4
  
  self.w            = self.sprites.idle:getWidth() * self.scale
  self.h            = self.sprites.idle:getHeight() * self.scale
  self.color        = colors[math.random(1, #colors)]
  self.x            = self.x or WIDTH
  self.y            = self.y or math.random(20, 150)
  self.timer        = 0
  self.moveSpeed    = self.moveSpeed or math.random(70, 200)
  self.shootTimer   = 0
  self.shootTime    = math.random(.5,3)
  self.hp           = self.hp or 3
  
  self.removed      = false
  setmetatable(self, Ghost)
  return self
end


function Ghost:getHitbox()
  return {x = self.x + 16, y = self.y + 16, w = self.w - 32, h = self.h - 32}
end


function Ghost:destroy()
  self.removed = true
end

function Ghost:update(dt)
  self.shootTimer = self.shootTimer + dt
  if self.shootTimer > self.shootTime then
    local bullet = Stone.new({x = self.x + self.w / 2, y = self.y + self.h / 2, xVel = math.random(-300, -500)})
    OBJECTS:add(bullet)
    self.shootTimer = 0
    self.shootTime = math.random(.5,3)
  end
  
  self.timer = self.timer + dt
  self.x = self.x - self.moveSpeed * dt
  self.y = self.y + (math.sin(self.timer * 8) * 60) * dt
end


function Ghost:draw()
  
    gfx.setColor(unpack(self.color))
    gfx.draw(self.sprites.idle, self.x, self.y, 0, self.scale, self.scale)
    
  
  if _showGhostHitbox == true then
    local box = self:getHitbox()
    gfx.setColor(0, 255, 0, 40)
    gfx.rectangle("fill", box.x, box.y, box.w, box.h)
  end
end

return Ghost
