
------------------
-- // Gnome \\ --
------------------
local Gnome = {}
Gnome.__index = Gnome


------------------------
-- // Declarations \\ --
------------------------

--Shortcuts
  lkid = love.keyboard.isDown
  gfx = love.graphics
  
--Locals
  local _showGnomeHitbox = false



function Gnome.new(o)
  local self = o or {}
  self.name         = "gnome"
  
  self.sprites = {
   
    run = gfx.newImage("gfx/gnome_run.png")
    
  }
  
  self.scale        = self.scale or 4
  
  self.w            = 32 * self.scale
  self.h            = 32 * self.scale
  
  self.x            = self.x or WIDTH
  self.y            = self.y or 320
  
  self.moveSpeed    = self.moveSpeed or math.random(85, 140)
  
  self.stoneTimer   = 0
  self.stoneRate    = math.random(.5, 1.25)
  
  self.animRun      = newAnimation(self.sprites.run, 32, 32, .1, 4)
  self.animRun:play()
  self.isJumping    = true
  
  self.removed      = false
  setmetatable(self, Gnome)
  return self
end


function Gnome:destroy()
  self.removed = true
end


function Gnome:getHitbox()
  return {x = self.x + 30, y = self.y + 8, w = self.w - 100, h = self.h - 16}
end


function Gnome:update(dt)
  self.x = self.x - self.moveSpeed * dt
  
  self.stoneTimer = self.stoneTimer + dt
  
  if self.stoneTimer > self.stoneRate then
    if math.random(1, 2) == 1 then
      local random = math.random(1, 6)
      if random == 1 then
        OBJECTS:add(Items.newPoison(self.x + self.w / 2, self.y + self.h / 2))
      elseif random == 2 or random == 4 then
        local stone = Stone.new({ x = self.x + self.w / 2, y = self.y + self.h / 2, mode = "rolling"})
        OBJECTS:add(stone)
      end
    end
    self.stoneTimer = 0
    self.stoneRate = math.random(1, 1.5)
  end
  
  self.animRun:update(dt)
end


function Gnome:draw()
  
    gfx.setColor(255, 255, 255, 255)
    self.animRun:draw(self.x, self.y, 0, self.scale, self.scale)
    
  
  if _showGnomeHitbox == true then
    local box = self:getHitbox()
    gfx.setColor(0, 255, 0, 40)
    gfx.rectangle("fill", box.x, box.y, box.w, box.h)
  end
end

return Gnome
