
------------------
-- // Items \\ --
------------------
local Items = {}
Items.__index = Items


------------------------
-- // Declarations \\ --
------------------------

--Shortcuts
  lkid = love.keyboard.isDown
  gfx = love.graphics
  
--Locals
  local _showItemsHitbox = false
  
  local _w, _h = 10, 10
  local _itemCount = 2



function Items.new(o)
  local self = o or {}
  self.name         = "item"
  
  self.sprites      = {
    energy = gfx.newImage("gfx/item_energydrink.png"),
    poison = gfx.newImage("gfx/item_poison.png"),
    burger = gfx.newImage("gfx/item_burger.png"),
    speed = gfx.newImage("gfx/item_speed.png")
  }
  
  self.scale        = self.scale or 3
  
  self.w            = 10
  self.h            = 10
  
  self.x            = self.x or 0
  self.y            = self.y or 0
  
  self.itemType     = self.itemType or "energy"
  
  self.xVel         = self.xVel or 80
  self.yVel         = self.yVel or 0
  
  self.gravity      = self.gravity or 500
  self.ground       = self.ground or HEIGHT - math.random(80, 100)
  
  self.removed      = false
  setmetatable(self, Items)
  return self
end


function Items:getHitbox()
  return {x = self.x, y = self.y, w = self.w, h = self.h}
end


function Items:destroy()
  self.removed = true
end

function Items:update(dt)
  self.x = self.x - self.xVel * dt
  
--Falling
  self.yVel = self.yVel + (self.gravity * dt)
  self.y = self.y + self.yVel * dt
  
  if self.y >= self.ground then
    self.yVel = 0
    self.y = self.ground
  end
  if self.y >= self.ground then
    self.xVel = 220
  else
    self.xVel = 80
  end
  
  if self.x < 0 - self.w then self:destroy() end
end


function Items:draw()
  gfx.setColor(255,255,255,255)
  gfx.draw(self.sprites[self.itemType], self.x, self.y, 0, self.scale, self.scale)
    
  
  if _showItemsHitbox == true then
    local box = self:getHitbox()
    gfx.setColor(0, 0, 255, 50)
    gfx.rectangle("fill", box.x, box.y, box.w, box.h)
  end
end

function Items.newGhostItem(x, y)
  rand = math.random(1, 3)
  
  itemType = "1337"
  
  if rand == 1 then itemType = "energy" end
  if rand == 2 then itemType = "burger" end
  if rand == 3 then itemType = "speed" end
  
  return Items.new({x = x, y = y, w = _w, h = _h, itemType = itemType, yVel = -350, gravity = 600})
end

function Items.newPoison(x, y)  
  return Items.new({x = x, y = y, w = _w, h = _h, itemType = "poison", yVel = -200, gravity = 600})
end

return Items
