
------------------
-- // MagicMissile \\ --
------------------
local MagicMissile = {}
MagicMissile.__index = MagicMissile


------------------------
-- // Declarations \\ --
------------------------

--Shortcuts
  lkid = love.keyboard.isDown
  gfx = love.graphics
  
--Locals
  local _showMagicMissileHitbox = false
  local soul


function MagicMissile.new(o)
  local self = o or {}
  self.name         = "MagicMissile"
  
  self.sprite       = self.sprite or gfx.newImage("gfx/reaper_bullet.png")
  
  self.scale        = self.scale or 4
  
  self.w            = self.sprite:getWidth() * self.scale
  self.h            = self.sprite:getHeight() * self.scale
  
  self.x            = self.x or 0
  self.y            = self.y or 0
  self.r            = self.r or 0
  
  self.xVel         = self.xVel or 630
  self.yVel         = self.yVel or 0
  
  self.timer        = 0
  
  self.fadeDelay    = .1
  self.fadeTime     = .2
  
  self.removed      = false
  setmetatable(self, MagicMissile)
  return self
end


function MagicMissile:destroy()
  self.removed = true
end

function MagicMissile:getHitbox()
  return {x = self.x, y = self.y, w = self.w, h = self.h}
end

function MagicMissile:update(dt)
--Timer
  self.timer = self.timer + dt
--Movement
  self.x = self.x + self.xVel * dt  
  self.y = self.y + self.yVel * dt

  --Kollision
    for i,v in ipairs(PLAYER.missiles.container) do  
      for ii,vv in ipairs(ENEMIES.container) do 
        if boxCollision(v:getHitbox(), vv:getHitbox()) then
          table.remove(PLAYER.missiles.container, i)
          table.remove(ENEMIES.container, ii)
          
          if vv.name == "gnome" then 
            soul = Souls.new({x = vv.x + vv.w / 2, y = vv.y + vv.h / 2, soulType = vv.name})
            
            KILLEDGNOMES = KILLEDGNOMES + 1
            PLAYER:addScore(50)
            
            --OBJECTS:add(Stone.newStone(vv.x, vv.y))
          else
            soul = Souls.new({x = vv.x + vv.w / 2, y = vv.y + vv.h / 2, soulType = vv.name, yVel = math.random(50, 70)})
            
            if math.random(1,2) == (1) then
              OBJECTS:add(Items.newGhostItem(vv.x, vv.y))
            end
            PLAYER:addScore(75)
            KILLEDGHOSTS = KILLEDGHOSTS + 1
          end
          OBJECTS:add(soul)
        end
      end
    end
  
  if self.x > WIDTH or self.timer > self.fadeDelay + self.fadeTime then
    self:destroy()
  end
end


function MagicMissile:draw()
    
    if self.timer > self.fadeDelay then
      gfx.setColor(255, 255, 255, -((self.timer - self.fadeDelay) / self.fadeTime) * 210)
    else
      gfx.setColor(255, 255, 255, 210)
    end
    
    gfx.draw(self.sprite, self.x, self.y, self.r, self.scale, self.scale)
      
    if _showMagicMissileHitbox == true then
      local box = self:getHitbox()
      gfx.setColor(255, 0, 0, 50)
      gfx.rectangle("fill", box.x, box.y, box.w, box.h)
    end

end

return MagicMissile
