
------------------
-- // Player \\ --
------------------
local Player = {}
Player.__index = Player


------------------------
-- // Declarations \\ --
------------------------
  
--Locals
local _showPlayerHitbox = false
local grav = 800


function Player.new(o)
  local self = o or {}
  self.name         = "Player"
  
  self.sprites = {
   
    run = gfx.newImage("gfx/reaper_run.png"),
    attack = gfx.newImage("gfx/reaper_attack.png"),
    
  }
  
  self.scale        = self.scale or 4
  
  self.w            = 32 * self.scale
  self.h            = 32 * self.scale
  
  self.x            = self.x or 110
  self.y            = self.y or 325
  
  self.ground       = self.ground or self.y
  
  self.state        = "run"
  
  self.spawnPoint   = { self.x, self.y }
  
  self.moveSpeedLeft= self.moveSpeedLeft or 200
  self.moveSpeedRight= self.moveSpeedRight or 120
  self.velY         = self.velY or 60
  self.velX         = self.velX or 60
  
  self.hunger       = self.hunger or 1000
  self.maxHunger    = self.maxHunger or 1000
  self.hungerLoss   = self.hungerLoss or 48
  self.score         = self.score or 0
  self.energy        = self.energy or 100
  self.maxEnergy     = self.maxenergy or 100
  self.energyGain    = self.energyGain or 10
  
  self.poisonTime   = 0
  self.poisonDuration = 4
  self.poisonDrain    = 98
  
  self.speedTimer    = 0
  self.speedDuration = 5
  self.speedTime    = 0
  
  self.missileCost  = self.missileCost or 4.5
  
  self.score        = 0
  
  self.missiles     = ObjectHandler.new()
  
  self.jumpImpulse  = -390
  
  self.keys         = self.keys or {right = "right", left = "left", shoot = " ", shootUpToggle = "up"}
  
  self.isJumping    = true
  self.stoneHit     = false
  self.isDead       = false
  self.animRun      = newAnimation(self.sprites.run, 32, 32, .14, 3)
  self.animRun:play()
  self.animAttack   = newAnimation(self.sprites.attack, 32, 32, .08, 4)
  self.animRun:setMode("bounce")
  self.animAttack:seek(2)
  self.delayTimer   = 0
  self.attackTimer  = 0
  self.attackRate   = .5
  self.delay        = 1
  self.hasAttacked  = false
  self.removed      = false
  setmetatable(self, Player)
  return self
end


function Player:destroy()
  self.isDead = true
  self.removed = true
end


function Player:addScore(v)
  self.score = self.score + v
end


function Player:respawn()
  self.hunger = self.maxHunger
  self.energy = self.maxenergy
  
  self.x, self.y = unpack(self.spawnPoint)
end


function Player:addPoints(v)
  if not self.removed then self.score = self.score + v end
end

function Player:getHitbox()
  return {x = self.x + 32, y = self.y + 16, w = self.w - 48, h = self.h - 32}
end


function Player:checkCollision(obj)
  local a = self:getHitbox()
  local b = obj:getHitbox()
  return a.x + a.w > b.x and a.x < b.x + b.w
  and a.y + a.h > b.y and a.y < b.y + b.h
end


function Player:update(dt)
  
  if not self.isDead then
  --Poison timer
    if self.poisonTime > 0 then self.poisonTime = self.poisonTime - dt end
    if self.speedTimer > 0 then self.speedTimer = self.speedTimer - dt end
    
  --Missiles
    self.missiles:update(dt)
  
  --Loose hunger over time
    if self.hunger > 0 and not MainMenu.isActive then 
      if self.poisonTime > 0 then
        self.hunger = self.hunger - self.poisonDrain * dt 
      else
        self.hunger = self.hunger - self.hungerLoss * dt 
      end
    end
    
  --Safety
    if self.hunger > self.maxHunger then self.hunger = self.maxHunger end
    
  --energy
    if self.energy < self.maxEnergy then self.energy = self.energy + self.energyGain * dt end
    
  --Soul collecting
    for i,v in ipairs(OBJECTS.container) do 
      if boxCollision(self:getHitbox(), v) and v.name == "soul" then
        if not v.collected then
          self:addHunger(v.soulValue)
          self:addEnergy(2)
          COLLECTEDSOULS = COLLECTEDSOULS + 1
          playSound("soul")
        end
        
        v.collected = true         
      end
      
      if boxCollision(self:getHitbox(), v:getHitbox()) and v.name == "stone" and not self.stoneHit then
        local sound = math.random(1, 2)
        SOUNDS.stonehit[sound]:stop()
        SOUNDS.stonehit[sound]:play()
        
        self.stoneHit = true
        self.velY = -200
        self.velX = 110
      end
    end
  
    -- Attack
  self.attackTimer = self.attackTimer - dt
  if lkid("c") and self.energy > self.missileCost and not self.isDead and not MainMenu.isActive and self.attackTimer <= 0 then
      self.state = "attack"
      self.energy = self.energy - self.missileCost
      self.attackTimer = self.attackRate
      self.hasAttacked = false
      playSound("shoot")
  end
  
  if self.state == "attack" and self.animAttack:getCurrentFrame() == 3 and not self.hasAttacked then
    if not lkid(self.keys.shootUpToggle) then
      missile = MagicMissile.new({x = PLAYER.x + PLAYER.w - 15, y = PLAYER.y + PLAYER.h / 2 - 20})
    else
      missile = MagicMissile.new({x = PLAYER.x + PLAYER.w - 15, y = PLAYER.y + PLAYER.h / 2 - 20, yVel = -630, r = math.rad(-45)})
    end
    self.missiles:add(missile)
    self.hasAttacked = true
  end
    
  --Dying
    if self.hunger < 0 or self.x < -self.w then
      self:destroy()
    end
  
  
  if not MainMenu.isActive then
    
      -- Not running fust enough !!
    self.x = self.x - self.velX * dt
    
    if not self.stoneHit then
      if lkid(self.keys.left) and self.x > 0 then
        if self.speedTimer > 0 then
          self.x = self.x - self.moveSpeedLeft * 1.5 * dt
        else
          self.x = self.x - self.moveSpeedLeft * dt
        end
        self.animRun:setSpeed(.75)
      elseif lkid(self.keys.right) and self.x and self.energy > 0 then
        self.animRun:setSpeed(2)
        if self.speedTimer > 0 then
          self.x = self.x + self.moveSpeedRight * 1.5 * dt
        else
          self.x = self.x + self.moveSpeedRight * dt
        end
        self.energy = self.energy - 19 * dt
      else
        self.animRun:setSpeed(1)
      end
    end
  end
  
  if self.state == "attack" then
    self.animAttack:update(dt)
    if self.animAttack:getCurrentFrame() == 1 then
      self.state = "run"
      self.animAttack:seek(2)
    end
  end
  
  --Update animations
  self.animRun:update(dt)
  
  -- Jumping
  self.velY = self.velY + (grav * dt)
  self.y = self.y + self.velY * dt
  if self.y >= self.ground then
    self.velY = 0
    self.velX = 40
    self.y = self.ground
    self.isJumping = false
    self.stoneHit = false
  end
  
end
end


function Player:draw()
  
  if not self.isDead then
    gfx.setColor(255, 255, 255, 255)
    if self.state == "run" then
      self.animRun:draw(self.x, self.y, 0, self.scale, self.scale)
    elseif self.state == "attack" then
      self.animAttack:draw(self.x, self.y, 0, self.scale, self.scale)
    end
    --gfx.draw(self.sprites.idle, self.x, self.y, 0, self.scale, self.scale)
  end 
  
  if _showPlayerHitbox == true then
    local box = self:getHitbox()
    gfx.setColor(0, 255, 0, 40)
    gfx.rectangle("fill", box.x, box.y, box.w, box.h)
  end
  
  --Missiles
    self.missiles:draw()
end


function Player:keypressed(key)
  
  if not MainMenu.isActive then
    if key == "x" and not self.isJumping then
      self.velY = self.jumpImpulse
      self.isJumping = true
      playSound("jump")
    end
  end
  
end

function Player:addHunger(amount)
  if self.hunger + amount > self.maxHunger then
    self.hunger = self.maxHunger
  else
    self.hunger = self.hunger + amount
  end
end

function Player:hungerDamage(amount)
  if self.hunger - amount < 0 then
    self.hunger = 0.05 * self.maxHunger
  else
    self.hunger = self.hunger - amount
  end
end

function Player:addEnergy(amount)
  if self.energy + amount > self.maxEnergy then
    self.energy = self.maxEnergy
  else
    self.energy = self.energy + amount
  end
end

function Player:poisonDebuff()
  self.poisonTime = self.poisonTime + self.poisonDuration
end

function Player:speedBuff()
  self.speedTimer = self.speedTimer + self.speedDuration
end

return Player
