
------------------
-- // Souls \\ --
------------------
local Souls = {}
Souls.__index = Souls


------------------------
-- // Declarations \\ --
------------------------

--Shortcuts
  lkid = love.keyboard.isDown
  gfx = love.graphics
  
--Locals
  local _showSoulsHitbox = false
  local grav = 20


function Souls.new(o)
  local self = o or {}
  self.name         = "soul"
  
  self.sprites      = {
    gnome = gfx.newImage("gfx/soul_gnome.png"),
    ghost = gfx.newImage("gfx/soul_ghost.png")
  }
  
  self.scale        = self.scale or 4
  
  self.w            = self.sprites.gnome:getWidth() * self.scale
  self.h            = self.sprites.gnome:getHeight() * self.scale
  
  self.x            = self.x or 0
  self.y            = self.y or 0
  
  self.alpha        = self.alpha or 255
  
  self.timer        = 0
  
  self.fadeDelay    = 2
  self.fadeTime     = 0.4
  
  self.soulType     = self.soulType or "default"
  
  self.soulValue    = self.soulValue or 100
  
  self.xVel         = self.xVel or math.random(-80, -110)
  self.yVel         = self.yVel or math.random(-15, -40)
  
  self.collected    = false
  
  self.collTimer    = 0
  self.collDuration = .5
  
  self.removed      = false
  setmetatable(self, Souls)
  return self
end


function Souls:destroy()
  self.removed = true
end


function Souls:getHitbox()
  return {x = self.x, y = self.y, w = self.w, h = self.h}
end


function Souls:update(dt)
--Timer
  if not self.collected then self.timer = self.timer + dt end
  if self.collected then self.collTimer = self.collTimer + dt end
  
  if not self.collected then
    self.x = self.x + (self.xVel + math.cos(self.timer * 8) * 60) * dt
    self.y = self.y + (self.yVel + math.sin(self.timer * 6) * 40) * dt  
    
    if self.timer > self.fadeDelay + self.fadeTime then
      self:destroy()
    end
  else
    if self.collTimer > self.collDuration then
      self:destroy()
      PLAYER:addScore(20)
    end
  end
  
end


function Souls:draw()
  
  if not self.collected then
    if self.timer > self.fadeDelay then
      gfx.setColor(255, 255, 255, -((self.timer - self.fadeDelay) / self.fadeTime) * 255)
    else
      gfx.setColor(255, 255, 255, 255)
    end
    
    gfx.draw(self.sprites[self.soulType], self.x, self.y, 0, self.scale, self.scale)
  else
    self.scale = self.scale + (self.collTimer / 75)
    gfx.setColor(255, 255, 255, -((self.collTimer / self.collDuration) * 255))
    gfx.draw(self.sprites[self.soulType], self.x + self.w * 2, self.y + self.h * 2, 0, self.scale, self.scale, self.w / 2, self.h / 2)
  end
    
  if _showSoulsHitbox == true then
    local box = self:getHitbox()
    gfx.setColor(255, 0, 0, 50)
    gfx.rectangle("fill", box.x, box.y, box.w, box.h)
  end
end

return Souls
