
----------------
-- // GAME \\ --
----------------
local Game = {paused = false}
PAUSED = false
------------------------
-- // Declarations \\ --
------------------------ 
local _debugging = true
local _image = {}
local moon = { x = love.graphics.getWidth() - 32, y = love.graphics.getHeight() - 256, scale = 4,  timer = 0}
--Shortcuts
  gfx = love.graphics
  
--Timer
  local _timer = {
    gnome = math.random(0, 2),
    ghost = math.random(0, 2)
  }
  
--Spawnrates
  local _rate = {
    gnome = math.random(1, 3.5),
    ghost = math.random(1, 3.5),
  }
  
--Zähler
  KILLEDGHOSTS = 0
  KILLEDGNOMES = 0
  COLLECTEDITEMS = 0
  COLLECTEDSOULS = 0
  
  

function Game:init()
  _image.back1 = gfx.newImage("gfx/level1_back1.png")
  _image.back2 = gfx.newImage("gfx/level1_back2.png")
  _image.center = gfx.newImage("gfx/level1_center.png")
  _image.front1 = gfx.newImage("gfx/level1_front1.png")
  _image.front2 = gfx.newImage("gfx/level1_front2.png")
  _image.moon   = gfx.newImage("gfx/moon.png")
  
  fog = Fog.new()
end


function Game:enter()
  --Reset counters
    KILLEDGHOSTS = 0
    KILLEDGNOMES = 0
    COLLECTEDITEMS = 0
    COLLECTEDSOULS = 0
    SECONDSPLAYED = 0
    MINUTESPLAYED = 0

  --Reset overlay
    --SceneManager.setOverlay("MainMenu", "draw")
  --OjectHandler
    OBJECTS = ObjectHandler.new()
    ENEMIES = ObjectHandler.new()
  --Player
    PLAYER = Player.new()
  --Stage
    background = Parallaxing.new({y = 340, sprites = {_image.back1,_image.back2}, xSpeed = -190})
    middleground = Parallaxing.new({y = 388, sprites = {_image.center}, xSpeed = -200})
    foreground = Parallaxing.new({y = 388, sprites = {_image.front1, _image.front2}, xSpeed = -210})
    
  --HUD
    HUD.reset()
end


function Game:leave()
  OBJECTS:destroy()
  ENEMIES:destroy()
end


function Game:update(dt)
  
  if not PAUSED then
--Sekunden
  if SceneManager.getOverlay() ~= "GameOver" then
    SECONDSPLAYED = SECONDSPLAYED + dt
  end
  
  if SECONDSPLAYED >= 60 then
    MINUTESPLAYED = MINUTESPLAYED + 1
    SECONDSPLAYED = 0
  end

  if not MainMenu.isActive then
--Timer
  for k,v in pairs(_timer) do
    _timer[k] = _timer[k] + dt
  end
  
--Spawn enemies
  if _timer.gnome > _rate.gnome then
    gnome = Gnome.new()
    ENEMIES:add(gnome)
    _rate.gnome = math.random(2, 4)
    _timer.gnome = 0
  end
  
  if _timer.ghost > _rate.ghost then
    ghost = Ghost.new()
    ENEMIES:add(ghost)
    _rate.ghost = math.random(2, 4)
    _timer.ghost = 0
  end
  
  OBJECTS:update(dt)
  ENEMIES:update(dt)
  
  end ---------------------------------- MainMenu.isActive end
  PLAYER:update(dt)
--Stage
  background:update(dt)
  foreground:update(dt)
  middleground:update(dt)
  
  fog:update(dt)
  
--HUD
  HUD.update(dt)
  
--Show GameOver screen
  if PLAYER.isDead and SceneManager.getOverlay() ~= "GameOver" then SceneManager.setOverlay("GameOver") end
  
--Items
  for i,v in ipairs(OBJECTS.container) do
    if v.name == "item" then
      if boxCollision(PLAYER:getHitbox(), v:getHitbox()) then
        if v.itemType == "energy" then
          PLAYER:addEnergy(27)
          PLAYER:addScore(25)
          playSound("energydrink")
        elseif v.itemType == "poison" then
          playSound("energydrink")
          PLAYER:poisonDebuff()
        elseif v.itemType == "burger" then
          playSound("energydrink")
          PLAYER:addHunger(80)
        elseif v.itemType == "speed" then
          playSound("energydrink")
          PLAYER:speedBuff()
        end
        table.remove(OBJECTS.container, i)
        COLLECTEDITEMS = COLLECTEDITEMS + 1
        PLAYER:addScore(15)
      end
    end
  end
  
  -- Moon movement
  moon.timer = moon.timer + dt
  moon.x = moon.x - 15 * dt
  moon.y = moon.y - (math.cos(moon.timer / 19)*10) * dt
  
  if moon.x < -512 then 
    moon.x = love.graphics.getWidth() - 32
    moon.y = love.graphics.getHeight() - 256
    moon.timer = 0
  end
  
  end ------------------------------ PAUSED END

end


function Game:draw()
  --Stage
  love.graphics.setColor(255, 255, 255, 200)
  love.graphics.draw(_image.moon, moon.x, moon.y, 0, 4, 4)
  fog:draw()
  
  background:draw()
  
  middleground:draw()
  
  OBJECTS:draw()
  ENEMIES:draw()
  
  PLAYER:draw()
    
  foreground:draw()
  
  if not MainMenu.isActive then
    
    if SceneManager.getOverlay() ~= "GameOver" then
      HUD.draw()
    end

    if PLAYER then 
      love.graphics.setColor(255, 255, 255, 255)
      love.graphics.setFont(FONT.medium)
      love.graphics.printf("Score: "..PLAYER.score, WIDTH / 2 - 400, 60, 800, "center")
    end
    
  end
  
  if PAUSED then
    gfx.setColor(255, 255, 255)    
    gfx.setFont(FONT.medium)    
    gfx.printf("Paused", WIDTH / 2 - 200, HEIGHT / 2, 400, "center")    
  end
  
end


function Game:keypressed(key, unicode)
  
  if not PAUSED then
  OBJECTS:keypressed(key, unicode)
  PLAYER:keypressed(key, unicode)
  if key == "r" then self:enter() end
  
  if key == "escape" then
    SceneManager.setOverlay("MainMenu")
  end
  
  if key == "return" and SceneManager.getOverlay() == "MainMenu" then
    SceneManager.setOverlay()
  end

  end

  if key == "p" then PAUSED = not PAUSED end

end

function Game:focus(f)
end

return Game
